#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <avr/io.h>
#include <avr/wdt.h>
#include <stdbool.h>

/* ISR occurring at 9.6 MHz / 256 = 37.5 kHz
 * OC0A: grid PWM drive
 * OC0B: boost drive
 */

ISR(TIM0_OVF_vect)
{
	static uint8_t div = 0;
	static uint16_t value = 0;

	/* 37.5 kHz / 146 = 256 Hz */
	if (++div < 146)
		return;

	div = 0;
	++value;
	value &= 0x1ff;

	if (value & 0x100)
		OCR0A = (value & 0xff) ^ 0xff;
	else
		OCR0A = value;
}

int main(void)
{
	PORTB	= 0xFC;
	DDRB	= 0x03;

	wdt_enable(WDTO_15MS);

	/* Timer 0:
	 * - clear OC0A and OC0B on compare match, set on top
	 * - fast PWM mode
	 * - no prescaling (9.6 MHz)
	 */

	TCCR0A	= _BV(COM0A1) | _BV(COM0B1) | _BV(WGM01) | _BV(WGM00);
	TCCR0B	= _BV(CS00);
	OCR0A	= 0;
	OCR0B	= 20;
	TIMSK0	= _BV(TOIE0);

	set_sleep_mode(SLEEP_MODE_IDLE);
	sei();

	for (;;)
	{
		sleep_mode();
		wdt_reset();
	}

	/* NOTREACHED */
	return 0;
}
